from os import path
import requests
from base import WrapperBase


BASE_URL = "https://esb.isc-seo.upenn.edu/8091/open_data/"
ENDPOINTS = {
    'SEARCH': BASE_URL + 'directory',
    'DETAILS': BASE_URL + 'directory_person_details',
}

class Directory(WrapperBase):
    """The client for the Directory. Used to make requests to the API.

    :param bearer: The user code for the API
    :param token: The password code for the API

    Usage::

      >>> from penn import Directory
      >>> d = Directory('MY_USERNAME_TOKEN', 'MY_PASSWORD_TOKEN')
    """

    def search(self, params):
        """Get a list of person objects for the given search params.

        :param params: Dictionary specifying the query parameters

        >>> people = d.search({'first_name': 'tobias', 'last_name': 'funke'})
        """
        return self._request(ENDPOINTS['SEARCH'], params)

    def detail_search(self, params):
        """Get a detailed list of person objects for the given search params.

        :param params:
            Dictionary specifying the query parameters

        >>> people_detailed = d.detail_search({'first_name': 'tobias', 'last_name': 'funke'})
        """

        response = self._request(ENDPOINTS['SEARCH'], params)
        result_data = []
        for person in response['result_data']:
            try:
                detail = self.person_details(person['person_id'])
            except ValueError:
                pass
            else:
                result_data.append(detail)

        response['result_data'] = result_data
        return response

    def person_details(self, person_id):
        """Get a detailed person object

        :param person_id:
            String corresponding to the person's id.

        >>> instructor = d.person('jhs878sfd03b38b0d463b16320b5e438')
        """
        return self._request(path.join(ENDPOINTS['DETAILS'], person_id))



from bs4 import BeautifulSoup
import requests
import re
import html5lib

class Laundry(object):
  """The client for Laundry. Used to make requests to the API.

    Usage::

      >>> from penn import Laundry
      >>> l = Laundry()
  """

  def __init__(self):
    pass

  def all_status(self):
    """Return names, hall numbers, and the washers/dryers available for all rooms in the system

    >>> all_laundry = l.all_status()
    """
    r = requests.get('https://www.laundryalert.com/cgi-bin/penn6389/LMPage?Login=True')
    print r.status_code
    if r.status_code == 200:
      parsed = BeautifulSoup(r.text)
      info_table = parsed.find_all('table')[2]

      hall_dict = dict()

      # This bit of code generates a dict of hallname->hall number by
      # parsing the link href of each room
      for link in info_table.find_all('a', class_='buttlink'):
        def get_hall_no(href):
          return int(re.search(r"Halls=(\d+)", href).group(1))
        hall_dict[link.get_text().strip()] = get_hall_no(link.get('href'))

      # Parse the table into the relevant data
      data = [filter(lambda x: len(x) > 0, [val.get_text().strip() for val in row.find_all('td')]) for row in info_table.find_all('tr')]

      # Remove the header row and all empty rows
      data_improved = (filter(lambda x: len(x) > 0, data))[1:]

      # Construct the final JSON
      laundry_dict = dict()
      for row in data_improved[1:]:
        room_dict = dict()
        room_dict['washers_available'] = int(row[1])
        room_dict['dryers_available'] = int(row[2])
        room_dict['washers_in_use'] = int(row[3])
        room_dict['dryers_in_use'] = int(row[4])
        room_dict['hall_no'] = hall_dict[row[0]]
        laundry_dict[row[0]] = room_dict
      return laundry_dict


  def hall_status(self, hall_no):
    """Return the status of each specific washer/dryer in a particular laundry room.

    :param hall_no:
       integer corresponding to the id number for the hall. Thus number is returned as part of the all_status call.

    >>> english_house = l.hall_status(2)
    """
    try:
      num = int(hall_no)
    except ValueError:
      raise ValueError("Room Number must be integer")

    r = requests.get('https://www.laundryalert.com/cgi-bin/penn6389/LMRoom?Halls=' + str(num))
    if r.status_code == 200:
      parsed = BeautifulSoup(r.text, 'html5lib')
      tables = parsed.find_all('table')
      hall_name = tables[2].get_text().strip()
      info_table = tables[4]
      data = [filter(lambda x: len(x) > 0, [val.get_text().strip() for val in row.find_all('td')]) for row in info_table.find_all('tr')]
      data_improved = (filter(lambda x: len(x) > 0, data))[1:]
      def toDict(data_row):
        d = dict()
        d['number'] = data_row[0]
        d['machine_type'] = data_row[1]
        d['available'] = data_row[2] == u'Available'
        if len(data_row) == 4:
          d['time_left'] = data_row[3]
        else:
          d['time_left'] = None
        return d

      return {'machines': map(toDict, data_improved), 'hall_name': hall_name}
    else:
      return {'error': 'The laundry api is currently unavailable.'}



__version__ = '1.1'

from registrar import Registrar
from directory import Directory
from dining import Dining
from transit import Transit
from news import News
from laundry import Laundry
from map import Map



from requests import get
import json

class WrapperBase(object):

    def __init__(self, bearer, token):
        self.bearer = bearer
        self.token = token

    @property
    def headers(self):
        """The HTTP headers needed for signed requests"""
        return {
            "Authorization-Bearer": self.bearer,
            "Authorization-Token": self.token,
        }

    def _request(self, url, params=None):
        """Make a signed request to the API, raise any API errors, and returning a tuple
        of (data, metadata)"""
        response = get(url, params=params, headers=self.headers)
        if response.status_code != 200:
            raise ValueError('Request to {} returned {}'.format(response.url, response.status_code))

        response = response.json()

        if response['service_meta']['error_text']:
            raise ValueError(response['service_meta']['error_text'])

        return response



from os import path
import requests
from base import WrapperBase


BASE_URL = "https://esb.isc-seo.upenn.edu/8091/open_data/"
ENDPOINTS = {
    'SEARCH': BASE_URL + 'news_events_maps'
}

class News(WrapperBase):
    """The client for the News Search API.

    :param bearer: The user code for the API
    :param token: The password code for the API

    Usage::

      >>> from penn import News
      >>> n = News('MY_USERNAME_TOKEN', 'MY_PASSWORD_TOKEN')
    """

    def search(self, keyword):
        """Return all news related to the provided query.

        :param keyword:
            The keyword for your news search

        >>> results = n.search('interview')
        """
        params = {
            "source": "news",
            "description": keyword
        }

        data = self._request(ENDPOINTS['SEARCH'], params)
        data['result_data'] = filter(lambda x: type(x) == dict, data['result_data'])
        return data



from setuptools import setup


setup(
    name='PennSDK',
    description='Python tools for building Penn-related applications',
    url='https://github.com/pennappslabs/penn-sdk-python',
    author='Penn Labs',
    author_email='pennappslabs@gmail.com',
    version='1.1',
    packages=['penn'],
    license='MIT',
    package_data = {
        # If any package contains *.txt or *.rst files, include them:
        '': ['*.txt', '*.rst'],
    },
    long_description=open('./README.rst').read(),
    install_requires=[
        'requests==2.4.3',
        'beautifulsoup4==4.3.2',
        'html5lib==0.999'
    ]
)



"""A module for consuming the Penn Dining API"""
from os import path
import requests
from base import WrapperBase
import json


BASE_URL = "https://esb.isc-seo.upenn.edu/8091/open_data/dining/"
ENDPOINTS = {
    'MENUS': BASE_URL + 'menus',
    'VENUES': BASE_URL + 'venues',
}

# Normalization for dining menu data
def normalize_weekly(data):
    if not "tblMenu" in data["result_data"]["Document"]:
        data["result_data"]["Document"]["tblMenu"] = []
    if isinstance(data["result_data"]["Document"]["tblMenu"], dict):
        data["result_data"]["Document"]["tblMenu"] = [data["result_data"]["Document"]["tblMenu"]]
    for day in data["result_data"]["Document"]["tblMenu"]:
        if isinstance(day["tblDayPart"], dict):
            day["tblDayPart"] = [day["tblDayPart"]]
        for meal in day["tblDayPart"]:
            if isinstance(meal["tblStation"], dict):
                meal["tblStation"] = [meal["tblStation"]]
            for station in meal["tblStation"]:
                if isinstance(station["tblItem"], dict):
                    station["tblItem"] = [station["tblItem"]]
    return data




class Dining(WrapperBase):
    """The client for the Registrar. Used to make requests to the API.

    :param bearer: The user code for the API
    :param token: The password code for the API

    Usage::

      >>> from penn import Dining
      >>> din = Dining('MY_USERNAME_TOKEN', 'MY_PASSWORD_TOKEN')
    """
    def venues(self):
        """Get a list of all venue objects.

          >>> venues = din.venues()
        """
        response = self._request(ENDPOINTS['VENUES'])
        # Normalize `dateHours` to array
        for venue in response["result_data"]["document"]["venue"]:
            if isinstance(venue.get("dateHours"), dict):
                venue["dateHours"] = [venue["dateHours"]]
        return response

    def menu_daily(self, building_id):
        """Get a menu object corresponding to the daily menu for the
        venue with building_id.

        :param building_id:
          A string representing the id of a building, e.g. "abc".


        >>> commons_today = din.menu_daily("593")
        """
        response = self._request(
            path.join(ENDPOINTS['MENUS'], 'daily', str(building_id))
        )
        # Normalize `tblDayPart` and `tblItem` to array
        meals = response["result_data"]["Document"]["tblMenu"]["tblDayPart"]
        if isinstance(meals, dict):
            response["result_data"]["Document"]["tblMenu"]["tblDayPart"] = [meals]
        for meal in response["result_data"]["Document"]["tblMenu"]["tblDayPart"]:
            for station in meal["tblStation"]:
                if isinstance(station["tblItem"], dict):
                    station["tblItem"] = [station["tblItem"]]
        return response

    def menu_weekly(self, building_id):
        """Get an array of menu objects corresponding to the weekly menu for the
        venue with building_id.

        :param building_id:
            A string representing the id of a building, e.g. "abc".

        >>> commons_week = din.menu_weekly("593")
        """
        response = self._request(path.join(ENDPOINTS['MENUS'], 'weekly', str(building_id)))
        return normalize_weekly(response)
